"""
This type stub file was generated by pyright.
"""

from functools import lru_cache
from typing import Any, Callable, Dict, List, Optional, Union
from huggingface_hub.utils._deprecation import _deprecate_method
from packaging import version
from ..models.auto import AutoProcessor
from ..utils import is_accelerate_available, is_torch_available

logger = ...
if is_torch_available():
    ...
if is_accelerate_available():
    ...
TOOL_CONFIG_FILE = ...
def get_repo_type(repo_id, repo_type=..., **hub_kwargs): # -> Literal['space', 'model']:
    ...

APP_FILE_TEMPLATE = ...
def validate_after_init(cls, do_validate_forward: bool = ...):
    ...

CONVERSION_DICT = ...
class Tool:
    """
    A base class for the functions used by the agent. Subclass this and implement the `__call__` method as well as the
    following class attributes:

    - **description** (`str`) -- A short description of what your tool does, the inputs it expects and the output(s) it
      will return. For instance 'This is a tool that downloads a file from a `url`. It takes the `url` as input, and
      returns the text contained in the file'.
    - **name** (`str`) -- A performative name that will be used for your tool in the prompt to the agent. For instance
      `"text-classifier"` or `"image_generator"`.
    - **inputs** (`Dict[str, Dict[str, Union[str, type]]]`) -- The dict of modalities expected for the inputs.
      It has one `type`key and a `description`key.
      This is used by `launch_gradio_demo` or to make a nice space from your tool, and also can be used in the generated
      description for your tool.
    - **output_type** (`type`) -- The type of the tool output. This is used by `launch_gradio_demo`
      or to make a nice space from your tool, and also can be used in the generated description for your tool.

    You can also override the method [`~Tool.setup`] if your tool as an expensive operation to perform before being
    usable (such as loading a model). [`~Tool.setup`] will be called the first time you use your tool, but not at
    instantiation.
    """
    name: str
    description: str
    inputs: Dict[str, Dict[str, Union[str, type]]]
    output_type: type
    @_deprecate_method(version="4.51.0", message="Switch to smolagents instead, with the same functionalities and similar API (https://huggingface.co/docs/smolagents/index)")
    def __init__(self, *args, **kwargs) -> None:
        ...
    
    @_deprecate_method(version="4.51.0", message="Switch to smolagents instead, with the same functionalities and similar API (https://huggingface.co/docs/smolagents/index)")
    def __init_subclass__(cls, **kwargs): # -> None:
        ...
    
    def validate_arguments(self, do_validate_forward: bool = ...): # -> None:
        ...
    
    def forward(self, *args, **kwargs):
        ...
    
    def __call__(self, *args, **kwargs):
        ...
    
    def setup(self): # -> None:
        """
        Overwrite this method here for any operation that is expensive and needs to be executed before you start using
        your tool. Such as loading a big model.
        """
        ...
    
    def save(self, output_dir): # -> None:
        """
        Saves the relevant code files for your tool so it can be pushed to the Hub. This will copy the code of your
        tool in `output_dir` as well as autogenerate:

        - a config file named `tool_config.json`
        - an `app.py` file so that your tool can be converted to a space
        - a `requirements.txt` containing the names of the module used by your tool (as detected when inspecting its
          code)

        You should only use this method to save tools that are defined in a separate module (not `__main__`).

        Args:
            output_dir (`str`): The folder in which you want to save your tool.
        """
        ...
    
    @classmethod
    def from_hub(cls, repo_id: str, token: Optional[str] = ..., **kwargs): # -> Any:
        """
        Loads a tool defined on the Hub.

        <Tip warning={true}>

        Loading a tool from the Hub means that you'll download the tool and execute it locally.
        ALWAYS inspect the tool you're downloading before loading it within your runtime, as you would do when
        installing a package using pip/npm/apt.

        </Tip>

        Args:
            repo_id (`str`):
                The name of the repo on the Hub where your tool is defined.
            token (`str`, *optional*):
                The token to identify you on hf.co. If unset, will use the token generated when running
                `huggingface-cli login` (stored in `~/.huggingface`).
            kwargs (additional keyword arguments, *optional*):
                Additional keyword arguments that will be split in two: all arguments relevant to the Hub (such as
                `cache_dir`, `revision`, `subfolder`) will be used when downloading the files for your tool, and the
                others will be passed along to its init.
        """
        ...
    
    def push_to_hub(self, repo_id: str, commit_message: str = ..., private: Optional[bool] = ..., token: Optional[Union[bool, str]] = ..., create_pr: bool = ...) -> str:
        """
        Upload the tool to the Hub.

        For this method to work properly, your tool must have been defined in a separate module (not `__main__`).
        For instance:
        ```
        from my_tool_module import MyTool
        my_tool = MyTool()
        my_tool.push_to_hub("my-username/my-space")
        ```

        Parameters:
            repo_id (`str`):
                The name of the repository you want to push your tool to. It should contain your organization name when
                pushing to a given organization.
            commit_message (`str`, *optional*, defaults to `"Upload tool"`):
                Message to commit while pushing.
            private (`bool`, *optional*):
                Whether to make the repo private. If `None` (default), the repo will be public unless the organization's default is private. This value is ignored if the repo already exists.
            token (`bool` or `str`, *optional*):
                The token to use as HTTP bearer authorization for remote files. If unset, will use the token generated
                when running `huggingface-cli login` (stored in `~/.huggingface`).
            create_pr (`bool`, *optional*, defaults to `False`):
                Whether or not to create a PR with the uploaded files or directly commit.
        """
        ...
    
    @staticmethod
    def from_space(space_id: str, name: str, description: str, api_name: Optional[str] = ..., token: Optional[str] = ...): # -> SpaceToolWrapper:
        """
        Creates a [`Tool`] from a Space given its id on the Hub.

        Args:
            space_id (`str`):
                The id of the Space on the Hub.
            name (`str`):
                The name of the tool.
            description (`str`):
                The description of the tool.
            api_name (`str`, *optional*):
                The specific api_name to use, if the space has several tabs. If not precised, will default to the first available api.
            token (`str`, *optional*):
                Add your token to access private spaces or increase your GPU quotas.
        Returns:
            [`Tool`]:
                The Space, as a tool.

        Examples:
        ```
        image_generator = Tool.from_space(
            space_id="black-forest-labs/FLUX.1-schnell",
            name="image-generator",
            description="Generate an image from a prompt"
        )
        image = image_generator("Generate an image of a cool surfer in Tahiti")
        ```
        ```
        face_swapper = Tool.from_space(
            "tuan2308/face-swap",
            "face_swapper",
            "Tool that puts the face shown on the first image on the second image. You can give it paths to images.",
        )
        image = face_swapper('./aymeric.jpeg', './ruth.jpg')
        ```
        """
        class SpaceToolWrapper(Tool):
            ...
        
        
    
    @staticmethod
    def from_gradio(gradio_tool): # -> GradioToolWrapper:
        """
        Creates a [`Tool`] from a gradio tool.
        """
        class GradioToolWrapper(Tool):
            ...
        
        
    
    @staticmethod
    def from_langchain(langchain_tool): # -> LangChainToolWrapper:
        """
        Creates a [`Tool`] from a langchain tool.
        """
        class LangChainToolWrapper(Tool):
            ...
        
        
    


DEFAULT_TOOL_DESCRIPTION_TEMPLATE = ...
def get_tool_description_with_args(tool: Tool, description_template: str = ...) -> str:
    ...

@lru_cache
def compile_jinja_template(template): # -> Template:
    ...

class PipelineTool(Tool):
    """
    A [`Tool`] tailored towards Transformer models. On top of the class attributes of the base class [`Tool`], you will
    need to specify:

    - **model_class** (`type`) -- The class to use to load the model in this tool.
    - **default_checkpoint** (`str`) -- The default checkpoint that should be used when the user doesn't specify one.
    - **pre_processor_class** (`type`, *optional*, defaults to [`AutoProcessor`]) -- The class to use to load the
      pre-processor
    - **post_processor_class** (`type`, *optional*, defaults to [`AutoProcessor`]) -- The class to use to load the
      post-processor (when different from the pre-processor).

    Args:
        model (`str` or [`PreTrainedModel`], *optional*):
            The name of the checkpoint to use for the model, or the instantiated model. If unset, will default to the
            value of the class attribute `default_checkpoint`.
        pre_processor (`str` or `Any`, *optional*):
            The name of the checkpoint to use for the pre-processor, or the instantiated pre-processor (can be a
            tokenizer, an image processor, a feature extractor or a processor). Will default to the value of `model` if
            unset.
        post_processor (`str` or `Any`, *optional*):
            The name of the checkpoint to use for the post-processor, or the instantiated pre-processor (can be a
            tokenizer, an image processor, a feature extractor or a processor). Will default to the `pre_processor` if
            unset.
        device (`int`, `str` or `torch.device`, *optional*):
            The device on which to execute the model. Will default to any accelerator available (GPU, MPS etc...), the
            CPU otherwise.
        device_map (`str` or `dict`, *optional*):
            If passed along, will be used to instantiate the model.
        model_kwargs (`dict`, *optional*):
            Any keyword argument to send to the model instantiation.
        token (`str`, *optional*):
            The token to use as HTTP bearer authorization for remote files. If unset, will use the token generated when
            running `huggingface-cli login` (stored in `~/.huggingface`).
        hub_kwargs (additional keyword arguments, *optional*):
            Any additional keyword argument to send to the methods that will load the data from the Hub.
    """
    pre_processor_class = AutoProcessor
    model_class = ...
    post_processor_class = AutoProcessor
    default_checkpoint = ...
    description = ...
    name = ...
    inputs = ...
    output_type = ...
    def __init__(self, model=..., pre_processor=..., post_processor=..., device=..., device_map=..., model_kwargs=..., token=..., **hub_kwargs) -> None:
        ...
    
    def setup(self): # -> None:
        """
        Instantiates the `pre_processor`, `model` and `post_processor` if necessary.
        """
        ...
    
    def encode(self, raw_inputs):
        """
        Uses the `pre_processor` to prepare the inputs for the `model`.
        """
        ...
    
    def forward(self, inputs):
        """
        Sends the inputs through the `model`.
        """
        ...
    
    def decode(self, outputs):
        """
        Uses the `post_processor` to decode the model output.
        """
        ...
    
    def __call__(self, *args, **kwargs):
        ...
    


def launch_gradio_demo(tool_class: Tool): # -> None:
    """
    Launches a gradio demo for a tool. The corresponding tool class needs to properly implement the class attributes
    `inputs` and `output_type`.

    Args:
        tool_class (`type`): The class of the tool for which to launch the demo.
    """
    ...

TOOL_MAPPING = ...
def load_tool(task_or_repo_id, model_repo_id=..., token=..., **kwargs): # -> Any:
    """
    Main function to quickly load a tool, be it on the Hub or in the Transformers library.

    <Tip warning={true}>

    Loading a tool means that you'll download the tool and execute it locally.
    ALWAYS inspect the tool you're downloading before loading it within your runtime, as you would do when
    installing a package using pip/npm/apt.

    </Tip>

    Args:
        task_or_repo_id (`str`):
            The task for which to load the tool or a repo ID of a tool on the Hub. Tasks implemented in Transformers
            are:

            - `"document_question_answering"`
            - `"image_question_answering"`
            - `"speech_to_text"`
            - `"text_to_speech"`
            - `"translation"`

        model_repo_id (`str`, *optional*):
            Use this argument to use a different model than the default one for the tool you selected.
        token (`str`, *optional*):
            The token to identify you on hf.co. If unset, will use the token generated when running `huggingface-cli
            login` (stored in `~/.huggingface`).
        kwargs (additional keyword arguments, *optional*):
            Additional keyword arguments that will be split in two: all arguments relevant to the Hub (such as
            `cache_dir`, `revision`, `subfolder`) will be used when downloading the files for your tool, and the others
            will be passed along to its init.
    """
    ...

def add_description(description): # -> Callable[..., Any]:
    """
    A decorator that adds a description to a function.
    """
    ...

class EndpointClient:
    def __init__(self, endpoint_url: str, token: Optional[str] = ...) -> None:
        ...
    
    @staticmethod
    def encode_image(image): # -> str:
        ...
    
    @staticmethod
    def decode_image(raw_image):
        ...
    
    def __call__(self, inputs: Optional[Union[str, Dict, List[str], List[List[str]]]] = ..., params: Optional[Dict] = ..., data: Optional[bytes] = ..., output_image: bool = ...) -> Any:
        ...
    


class ToolCollection:
    """
    Tool collections enable loading all Spaces from a collection in order to be added to the agent's toolbox.

    > [!NOTE]
    > Only Spaces will be fetched, so you can feel free to add models and datasets to your collection if you'd
    > like for this collection to showcase them.

    Args:
        collection_slug (str):
            The collection slug referencing the collection.
        token (str, *optional*):
            The authentication token if the collection is private.

    Example:

    ```py
    >>> from transformers import ToolCollection, ReactCodeAgent

    >>> image_tool_collection = ToolCollection(collection_slug="huggingface-tools/diffusion-tools-6630bb19a942c2306a2cdb6f")
    >>> agent = ReactCodeAgent(tools=[*image_tool_collection.tools], add_base_tools=True)

    >>> agent.run("Please draw me a picture of rivers and lakes.")
    ```
    """
    def __init__(self, collection_slug: str, token: Optional[str] = ...) -> None:
        ...
    


def tool(tool_function: Callable) -> Tool:
    """
    Converts a function into an instance of a Tool subclass.

    Args:
        tool_function: Your function. Should have type hints for each input and a type hint for the output.
        Should also have a docstring description including an 'Args:' part where each argument is described.
    """
    class SpecificTool(Tool):
        ...
    
    

